from .docker import get_files_from_docker_archive
from .scannable import Commit, File, Files, Result, ScanCollection


__all__ = [
    "Commit",
    "File",
    "Files",
    "Result",
    "ScanCollection",
    "get_files_from_docker_archive",
]



import os
import sys
from typing import List, Optional, Tuple

import click

from ggshield.dev_scan import scan_commit_range
from ggshield.output import TextOutputHandler
from ggshield.scan import Commit, ScanCollection
from ggshield.utils import EMPTY_SHA, EMPTY_TREE, SupportedScanMode, handle_exception

from .git_shell import check_git_dir, get_list_commit_SHA


@click.command()
@click.argument("precommit_args", nargs=-1, type=click.UNPROCESSED)
@click.pass_context
def precommit_cmd(
    ctx: click.Context, precommit_args: List[str]
) -> int:  # pragma: no cover
    """
    scan as a pre-commit git hook.
    """
    config = ctx.obj["config"]
    output_handler = TextOutputHandler(
        show_secrets=config.show_secrets, verbose=config.verbose, output=None
    )
    try:
        check_git_dir()
        results = Commit(exclusion_regexes=ctx.obj["exclusion_regexes"]).scan(
            client=ctx.obj["client"],
            cache=ctx.obj["cache"],
            matches_ignore=config.matches_ignore,
            all_policies=config.all_policies,
            verbose=config.verbose,
            mode_header=SupportedScanMode.PRE_COMMIT.value,
            banlisted_detectors=config.banlisted_detectors,
        )

        return output_handler.process_scan(
            ScanCollection(id="cached", type="pre-commit", results=results)
        )
    except Exception as error:
        return handle_exception(error, config.verbose)


def collect_from_stdin() -> Tuple[str, str]:
    """
    Collect pre-commit variables from stdin.
    """
    prepush_input = sys.stdin.read().split()
    if len(prepush_input) < 4:
        # Then it's either a tag or a deletion event
        local_commit = EMPTY_SHA
        remote_commit = EMPTY_SHA
    else:
        local_commit = prepush_input[1].strip()
        remote_commit = prepush_input[3].strip()

    return (local_commit, remote_commit)


def collect_from_precommit_env() -> Tuple[Optional[str], Optional[str]]:
    """
    Collect from pre-commit framework environment.
    """
    # pre-commit framework <2.2.0
    local_commit = os.getenv("PRE_COMMIT_SOURCE", None)
    remote_commit = os.getenv("PRE_COMMIT_ORIGIN", None)

    if local_commit is None or remote_commit is None:
        # pre-commit framework >=2.2.0
        local_commit = os.getenv("PRE_COMMIT_FROM_REF", None)
        remote_commit = os.getenv("PRE_COMMIT_TO_REF", None)

    return (local_commit, remote_commit)


@click.command()
@click.argument("prepush_args", nargs=-1, type=click.UNPROCESSED)
@click.pass_context
def prepush_cmd(ctx: click.Context, prepush_args: List[str]) -> int:  # pragma: no cover
    """
    scan as a pre-push git hook.
    """
    config = ctx.obj["config"]

    local_commit, remote_commit = collect_from_precommit_env()
    if local_commit is None or remote_commit is None:
        local_commit, remote_commit = collect_from_stdin()

    if local_commit == EMPTY_SHA:
        click.echo("Deletion event or nothing to scan.")
        return 0

    if remote_commit == EMPTY_SHA:
        click.echo(
            f"New tree event. Scanning last {config.max_commits_for_hook} commits."
        )
        before = EMPTY_TREE
        after = local_commit
        cmd_range = (
            f"--max-count={config.max_commits_for_hook+1} {EMPTY_TREE} {local_commit}"
        )
    else:
        before = remote_commit
        after = local_commit
        cmd_range = f"--max-count={config.max_commits_for_hook+1} {remote_commit}...{local_commit}"  # noqa

    commit_list = get_list_commit_SHA(cmd_range)

    if not commit_list:
        click.echo(
            "Unable to get commit range.\n"
            f"  before: {before}\n"
            f"  after: {after}\n"
            "Skipping pre-push hook\n"
        )
        return 0

    if len(commit_list) > config.max_commits_for_hook:
        click.echo(
            f"Too many commits. Scanning last {config.max_commits_for_hook} commits\n"
        )
        commit_list = commit_list[-config.max_commits_for_hook :]

    if config.verbose:
        click.echo(f"Commits to scan: {len(commit_list)}")

    try:
        check_git_dir()
        return scan_commit_range(
            client=ctx.obj["client"],
            cache=ctx.obj["cache"],
            commit_list=commit_list,
            output_handler=ctx.obj["output_handler"],
            verbose=config.verbose,
            exclusion_regexes=ctx.obj["exclusion_regexes"],
            matches_ignore=config.matches_ignore,
            all_policies=config.all_policies,
            scan_id=" ".join(commit_list),
            mode_header=SupportedScanMode.PRE_PUSH.value,
            banlisted_detectors=config.banlisted_detectors,
        )
    except Exception as error:
        return handle_exception(error, config.verbose)



import subprocess
import tempfile
from pathlib import Path
from typing import Any, Dict, Iterable, List, Optional, Set

import click
from pygitguardian.client import GGClient

from ggshield.config import Cache
from ggshield.output import OutputHandler
from ggshield.scan import ScanCollection, get_files_from_docker_archive
from ggshield.utils import SupportedScanMode, handle_exception


# bailout if docker command takes longer than 6 minutes
DOCKER_COMMAND_TIMEOUT = 360


class DockerArchiveCreationError(Exception):
    pass


def docker_pull_image(image_name: str, timeout: int) -> None:
    """
    Pull docker image and raise exception on timeout or failed to find image

    Timeout after `timeout` seconds.
    """
    command = ["docker", "pull", image_name]
    try:
        subprocess.run(
            command,
            check=True,
            timeout=timeout,
        )
    except subprocess.CalledProcessError:
        raise click.ClickException(f'Image "{image_name}" not found')
    except subprocess.TimeoutExpired:
        raise click.ClickException('Command "{}" timed out'.format(" ".join(command)))


def docker_save_to_tmp(image_name: str, destination_path: Path, timeout: int) -> None:
    """
    Do a `docker save <image_name> -o <destination_path>`

    Limit docker commands to run at most `timeout` seconds.
    """
    command = ["docker", "save", image_name, "-o", str(destination_path)]

    try:
        click.echo("Saving docker image... ", nl=False)
        subprocess.run(
            command,
            check=True,
            stderr=subprocess.PIPE,
            timeout=timeout,
        )
        click.echo("OK")
    except subprocess.CalledProcessError as exc:
        err_string = str(exc.stderr)
        if "No such image" in err_string or "reference does not exist" in err_string:
            click.echo("need to download image first")
            docker_pull_image(image_name, timeout)

            docker_save_to_tmp(image_name, destination_path, timeout)
        raise click.ClickException(
            f"Unable to save docker archive:\nError: {err_string}"
        )
    except subprocess.TimeoutExpired:
        raise click.ClickException('Command "{}" timed out'.format(" ".join(command)))


def docker_scan_archive(
    archive: Path,
    client: GGClient,
    cache: Cache,
    verbose: bool,
    matches_ignore: Iterable[str],
    all_policies: bool,
    scan_id: str,
    banlisted_detectors: Optional[Set[str]] = None,
) -> ScanCollection:
    files = get_files_from_docker_archive(archive)
    with click.progressbar(length=len(files.files), label="Scanning") as progressbar:

        def update_progress(chunk: List[Dict[str, Any]]) -> None:
            progressbar.update(len(chunk))

        results = files.scan(
            client=client,
            cache=cache,
            matches_ignore=matches_ignore,
            all_policies=all_policies,
            verbose=verbose,
            mode_header=SupportedScanMode.DOCKER.value,
            on_file_chunk_scanned=update_progress,
            banlisted_detectors=banlisted_detectors,
        )

    return ScanCollection(id=scan_id, type="scan_docker_archive", results=results)


@click.command()
@click.option(
    "--docker-timeout",
    type=click.INT,
    default=DOCKER_COMMAND_TIMEOUT,
    help="Timeout for Docker commands.",
    metavar="SECONDS",
    show_default=True,
)
@click.argument("name", nargs=1, type=click.STRING, required=True)
@click.pass_context
def docker_name_cmd(ctx: click.Context, name: str, docker_timeout: int) -> int:
    """
    scan a docker image <NAME>.

    ggshield will try to pull the image if it's not available locally.
    """

    with tempfile.TemporaryDirectory(suffix="ggshield") as temporary_dir:
        config = ctx.obj["config"]
        output_handler: OutputHandler = ctx.obj["output_handler"]

        try:
            archive = Path(temporary_dir) / "archive.tar"
            docker_save_to_tmp(name, archive, docker_timeout)

            scan = docker_scan_archive(
                archive=archive,
                client=ctx.obj["client"],
                cache=ctx.obj["cache"],
                verbose=config.verbose,
                matches_ignore=config.matches_ignore,
                all_policies=config.all_policies,
                scan_id=name,
                banlisted_detectors=config.banlisted_detectors,
            )

            return output_handler.process_scan(scan)
        except Exception as error:
            return handle_exception(error, config.verbose)


@click.command(hidden=True)
@click.argument(
    "archive", nargs=1, type=click.Path(exists=True, resolve_path=True), required=True
)
@click.pass_context
def docker_archive_cmd(
    ctx: click.Context,
    archive: Path,
) -> int:  # pragma: no cover
    """
    scan a docker archive <ARCHIVE> without attempting to save or pull the image.

    Hidden command `ggshield scan docker-archive`
    """
    config = ctx.obj["config"]
    output_handler: OutputHandler = ctx.obj["output_handler"]

    try:
        scan = docker_scan_archive(
            archive=archive,
            client=ctx.obj["client"],
            cache=ctx.obj["cache"],
            verbose=config.verbose,
            matches_ignore=config.matches_ignore,
            all_policies=config.all_policies,
            scan_id=str(archive),
            banlisted_detectors=config.banlisted_detectors,
        )

        return output_handler.process_scan(scan)
    except Exception as error:
        return handle_exception(error, config.verbose)



import concurrent.futures
import re
from typing import Any, Callable, Dict, Iterable, List, NamedTuple, Optional, Set

import click
from pygitguardian import GGClient
from pygitguardian.config import MULTI_DOCUMENT_LIMIT
from pygitguardian.models import ScanResult

from ggshield.config import CPU_COUNT, MAX_FILE_SIZE, Cache
from ggshield.filter import (
    is_filepath_excluded,
    remove_ignored_from_result,
    remove_results_from_banlisted_detectors,
)
from ggshield.git_shell import GIT_PATH, shell
from ggshield.text_utils import STYLE, format_text
from ggshield.utils import REGEX_HEADER_INFO, Filemode

from .scannable_errors import handle_scan_error


class Result(NamedTuple):
    """
    Return model for a scan which zips the information
    between the Scan result and its input content.
    """

    content: str  # Text content scanned
    filemode: Filemode  # Filemode (useful for commits)
    filename: str  # Filename of content scanned
    scan: ScanResult  # Result of content scan


class ScanCollection(NamedTuple):
    id: str
    type: str
    results: Optional[List[Result]] = None
    scans: Optional[List[Any]] = None
    # Foward references are not support in mypy for NamedTuples
    # Correct typing would be Union
    optional_header: Optional[str] = None  # To be printed in Text Output
    extra_info: Optional[Dict[str, str]] = None  # To be included in JSON Output

    @property
    def scans_with_results(self) -> List[Any]:
        if self.scans:
            return [scan for scan in self.scans if scan.results]
        return []


class File:
    """Class representing a simple file."""

    def __init__(self, document: str, filename: str, filesize: Optional[int] = None):
        self.document = document
        self.filename = filename
        self.filemode = Filemode.FILE
        self.filesize = filesize if filesize else len(self.document.encode("utf-8"))

    @property
    def scan_dict(self) -> Dict[str, Any]:
        """Return a payload compatible with the scanning API."""
        return {
            "filename": self.filename
            if len(self.filename) <= 256
            else self.filename[-255:],
            "document": self.document,
            "filemode": self.filemode,
        }


class CommitFile(File):
    """Class representing a commit file."""

    def __init__(
        self,
        document: str,
        filename: str,
        filemode: Filemode,
        filesize: Optional[int] = None,
    ):
        super().__init__(document, filename, filesize)
        self.filemode = filemode


class Files:
    """
    Files is a list of files. Useful for directory scanning.
    """

    def __init__(self, files: List[File]):
        self._files = {entry.filename: entry for entry in files}

    @property
    def files(self) -> Dict[str, File]:
        return self._files

    @property
    def scannable_list(self) -> List[Dict[str, Any]]:
        return [entry.scan_dict for entry in self.files.values()]

    def scan(
        self,
        client: GGClient,
        cache: Cache,
        matches_ignore: Iterable[str],
        all_policies: bool,
        verbose: bool,
        mode_header: str,
        banlisted_detectors: Optional[Set[str]] = None,
        on_file_chunk_scanned: Callable[
            [List[Dict[str, Any]]], None
        ] = lambda chunk: None,
    ) -> List[Result]:
        cache.purge()
        scannable_list = self.scannable_list
        results = []
        chunks = []
        for i in range(0, len(scannable_list), MULTI_DOCUMENT_LIMIT):
            chunks.append(scannable_list[i : i + MULTI_DOCUMENT_LIMIT])

        with concurrent.futures.ThreadPoolExecutor(
            max_workers=min(CPU_COUNT, 4), thread_name_prefix="content_scan"
        ) as executor:
            future_to_scan = {
                executor.submit(
                    client.multi_content_scan,
                    chunk,
                    {"mode": mode_header},
                ): chunk
                for chunk in chunks
            }

            for future in concurrent.futures.as_completed(future_to_scan):
                chunk = future_to_scan[future]
                on_file_chunk_scanned(chunk)

                scan = future.result()
                if not scan.success:
                    handle_scan_error(scan, chunk)
                    continue
                for index, scanned in enumerate(scan.scan_results):
                    remove_ignored_from_result(scanned, all_policies, matches_ignore)
                    remove_results_from_banlisted_detectors(
                        scanned, banlisted_detectors
                    )
                    if scanned.has_policy_breaks:
                        for policy_break in scanned.policy_breaks:
                            cache.add_found_policy_break(
                                policy_break, chunk[index]["filename"]
                            )
                        results.append(
                            Result(
                                content=chunk[index]["document"],
                                scan=scanned,
                                filemode=chunk[index]["filemode"],
                                filename=chunk[index]["filename"],
                            )
                        )
        cache.save()
        return results


class CommitInformation(NamedTuple):
    author: str
    email: str
    date: str


class Commit(Files):
    """
    Commit represents a commit which is a list of commit files.
    """

    def __init__(
        self, sha: Optional[str] = None, exclusion_regexes: Set[re.Pattern] = set()
    ):
        self.sha = sha
        self._patch: Optional[str] = None
        self._files = {}
        self.exclusion_regexes = exclusion_regexes
        self._info: Optional[CommitInformation] = None

    @property
    def info(self) -> CommitInformation:
        if self._info is None:
            m = REGEX_HEADER_INFO.search(self.patch)

            if m is None:
                self._info = CommitInformation("unknown", "", "")
            else:
                self._info = CommitInformation(**m.groupdict())

        return self._info

    @property
    def optional_header(self) -> str:
        """Return the formatted patch."""
        return (
            format_text(f"\ncommit {self.sha}\n", STYLE["commit_info"])
            + f"Author: {self.info.author} <{self.info.email}>\n"
            + f"Date: {self.info.date}\n"
        )

    @property
    def patch(self) -> str:
        """Get the change patch for the commit."""
        if self._patch is None:
            if self.sha:
                self._patch = shell([GIT_PATH, "show", self.sha])
            else:
                self._patch = shell([GIT_PATH, "diff", "--cached"])

        return self._patch

    @property
    def files(self) -> Dict[str, File]:
        if not self._files:
            self._files = {entry.filename: entry for entry in list(self.get_files())}

        return self._files

    @staticmethod
    def get_filename(line: str) -> str:
        """
        Get the file path from the line patch

        Example: line = "a/filename.txt b/filename.txt"
        """
        return line.split(" ")[1][2:]

    @staticmethod
    def get_filemode(line: str) -> Filemode:
        """
        Get the file mode from the line patch (new, modified or deleted)

        :raise: Exception if filemode is not detected
        """
        if line.startswith("index"):
            return Filemode.MODIFY
        if line.startswith("similarity"):
            return Filemode.RENAME
        if line.startswith("new"):
            return Filemode.NEW
        if line.startswith("deleted"):
            return Filemode.DELETE
        if line.startswith("old"):
            return Filemode.PERMISSION_CHANGE

        raise click.ClickException(f"Filemode is not detected:{line}")

    def get_files(self) -> Iterable[CommitFile]:
        """
        Format the diff into files and extract the patch for each one of them.

        Example :
            diff --git a/test.txt b/test.txt\n
            new file mode 100644\n
            index 0000000..b80e3df\n
            --- /dev/null\n
            +++ b/test\n
            @@ -0,0 +1,28 @@\n
            +this is a test patch\n
        """
        list_diff = re.split(r"^diff --git ", self.patch, flags=re.MULTILINE)[1:]

        for diff in list_diff:
            lines = diff.split("\n")

            filename = self.get_filename(lines[0])
            if is_filepath_excluded(filename, self.exclusion_regexes):
                continue

            filemode = self.get_filemode(lines[1])
            document = "\n".join(lines[filemode.start :])
            file_size = len(document.encode("utf-8"))
            if file_size > MAX_FILE_SIZE * 0.90:
                continue

            if document:
                yield CommitFile(document, filename, filemode, file_size)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_api_status[test_health_check-False] 1'] = '''\x1b[97m\x1b[1m\x1b[22mAPI URL:\x1b[0m https://api.gitguardian.com/
\x1b[97m\x1b[1m\x1b[22mStatus:\x1b[0m \x1b[32m\x1b[22m\x1b[22mhealthy\x1b[0m
\x1b[97m\x1b[1m\x1b[22mApp version:\x1b[0m 1.32.0-rc.2
\x1b[97m\x1b[1m\x1b[22mSecrets engine version:\x1b[0m 2.56.0

'''

snapshots['test_api_status[test_health_check-True] 1'] = '''{"detail": "Valid API key.", "status_code": 200, "app_version": "1.32.0-rc.2", "secrets_engine_version": "2.56.0"}
'''

snapshots['test_api_status[test_health_check_error-False] 1'] = '''\x1b[97m\x1b[1m\x1b[22mAPI URL:\x1b[0m https://api.gitguardian.com/
\x1b[97m\x1b[1m\x1b[22mStatus:\x1b[0m \x1b[32m\x1b[22m\x1b[22mhealthy\x1b[0m
\x1b[97m\x1b[1m\x1b[22mApp version:\x1b[0m 1.32.0-rc.2
\x1b[97m\x1b[1m\x1b[22mSecrets engine version:\x1b[0m 2.56.0

'''

snapshots['test_quota[quota-False] 1'] = '''Quota available: \x1b[32m\x1b[22m\x1b[22m806\x1b[0m
Quota used in the last 30 days: 194
Total Quota of the workspace: 1000

'''

snapshots['test_quota[quota-True] 1'] = '''{"count": 194, "limit": 1000, "remaining": 806, "since": "2021-11-02"}
'''

snapshots['test_quota[quota_half_remaining-False] 1'] = '''Quota available: \x1b[32m\x1b[22m\x1b[22m806\x1b[0m
Quota used in the last 30 days: 194
Total Quota of the workspace: 1000

'''

snapshots['test_quota[quota_low_remaining-False] 1'] = '''Quota available: \x1b[32m\x1b[22m\x1b[22m806\x1b[0m
Quota used in the last 30 days: 194
Total Quota of the workspace: 1000

'''



from pathlib import Path
from typing import Dict

import pytest

from ggshield.scan.docker import (
    InvalidDockerArchiveException,
    _get_config,
    _should_scan_layer,
    get_files_from_docker_archive,
)


DOCKER_EXAMPLE_PATH = Path(__file__).parent.parent / "data" / "docker-example.tar.xz"
DOCKER__INCOMPLETE_MANIFEST_EXAMPLE_PATH = (
    Path(__file__).parent.parent / "data" / "docker-incomplete-manifest-example.tar.xz"
)


class ManifestMock:
    def read(self, amount: int = None) -> bytes:
        return '[{"Config": "8b907fee27ad927c595fcf873c8256796cab27e7a3fb4bf3952308a76ad791c4.json"}]'.encode(  # noqa: E501
            "utf-8"
        )


class TarMock:
    def __init__(self, members: Dict[str, str], *args, **kwargs):
        self.members = members

    def extractfile(self, member: str):
        if "8b907fee27ad927c595fcf873c8256796cab27e7a3fb4bf3952308a76ad791c4" in member:
            return None
        return self.members.get(member, None)

    def getmember(self, member: str):
        return member if self.members.get(member, None) else None


class TestDockerScan:
    @pytest.mark.parametrize(
        ["op", "want"],
        [
            pytest.param(
                "/bin/sh -c #(nop) COPY dir:xxx in / ",
                True,
            ),
            pytest.param("/bin/sh -c #(nop) ADD dir:xxx in / ", True),
            pytest.param(
                '/bin/sh -c #(nop)  CMD ["/usr/bin/bash"',
                False,
            ),
        ],
    )
    def test_should_scan_layer(self, op: str, want: bool):
        assert _should_scan_layer({"created_by": op}) is want

    @pytest.mark.parametrize(
        ["members", "match"],
        [
            pytest.param({}, "No manifest file found."),
            pytest.param(
                {"manifest.json": ManifestMock()},
                "No config file found.",
            ),
            pytest.param(
                {
                    "manifest.json": ManifestMock(),
                    "8b907fee27ad927c595fcf873c8256796cab27e7a3fb4bf3952308a76ad791c4.json": "layer file",  # noqa: E501
                },  # noqa: E501
                "Config file could not be extracted.",
            ),
        ],
    )
    def test_get_config(self, members, match):
        tarfile = TarMock(members)
        with pytest.raises(InvalidDockerArchiveException, match=match):
            _get_config(tarfile)

    @pytest.mark.parametrize(
        "image_path", [DOCKER_EXAMPLE_PATH, DOCKER__INCOMPLETE_MANIFEST_EXAMPLE_PATH]
    )
    def test_get_files_from_docker_archive(self, image_path: Path):
        files = get_files_from_docker_archive(image_path)

        expected_files = {
            "Dockerfile or build-args": None,  # noqa: E501
            image_path
            / "64a345482d74ea1c0699988da4b4fe6cda54a2b0ad5da49853a9739f7a7e5bbc/layer.tar/app/file_one": "Hello, I am the first file!\n",  # noqa: E501
            image_path
            / "2d185b802fb3c2e6458fe1ac98e027488cd6aedff2e3d05eb030029c1f24d60f/layer.tar/app/file_three.sh": "echo Life is beautiful.\n",  # noqa: E501
            image_path
            / "2d185b802fb3c2e6458fe1ac98e027488cd6aedff2e3d05eb030029c1f24d60f/layer.tar/app/file_two.py": """print("Hi! I'm the second file but I'm happy.")\n""",  # noqa: E501
        }

        assert set(files.files) == {str(file_path) for file_path in expected_files}

        for file_path, expected_content in expected_files.items():
            file = files.files[str(file_path)]
            assert expected_content is None or file.document == expected_content



import os
import re
import traceback
from enum import Enum
from typing import Iterable, List, NamedTuple, Optional

import click
import urllib3
from pygitguardian import GGClient
from pygitguardian.models import Match
from requests import Session

from .text_utils import Line, LineCategory


REGEX_PATCH_HEADER = re.compile(
    r"^(?P<line_content>@@ -(?P<pre_index>\d+),?\d* \+(?P<post_index>\d+),?\d* @@(?: .+)?)"  # noqa
)

# Source: https://github.com/jonschlinkert/is-git-url MIT LICENSE
REGEX_GIT_URL = re.compile(
    r"(?:git|ssh|https?|git@[-\w.]+):(\/\/)?(.*?)(\.git)(\/?|\#[-\d\w._]+?)$"
)

REGEX_HEADER_INFO = re.compile(
    r"Author:\s(?P<author>.+?)\ <(?P<email>.+?)>\nDate:\s+(?P<date>.+)?\n"
)

EMPTY_SHA = "0000000000000000000000000000000000000000"
EMPTY_TREE = "4b825dc642cb6eb9a060e54bf8d69288fbee4904"

# GitHub timeouts every pre-receive hook after 5s with an error.
# We try and anticipate that so we can control the return code
PRERECEIVE_TIMEOUT = 4.5

IGNORED_DEFAULT_PATTERNS = [
    "**/.git/**/*",
    "**/.pytest_cache/**/*",
    "**/.mypy_cache/**/*",
    "**/.venv/**/*",
    "**/.eggs/**/*",
    "**/.eggs-info/**/*",
    "**/vendor/**/*",
    "**/vendors/**/*",
    "**/node_modules/**/*",
    "top-1000.txt*",
    "**/*.storyboard*",
    "**/*.xib",
    "**/*.mdx*",
    "**/*.sops",
]


class Filemode(Enum):
    """
    Enum class for git filemode.

    Attributes:
        start (int): The first line to read in this filemode scenario
        mode  (str): The string filemode
    """

    MODIFY = (4, "modified file")
    DELETE = (5, "deleted file")
    NEW = (5, "new file")
    RENAME = (7, "renamed file")
    PERMISSION_CHANGE = (7, "changed permissions")
    FILE = (0, "file")

    def __init__(self, start: int, mode: str):
        self.start = start
        self.mode = mode


def get_lines_from_content(
    content: str, filemode: Filemode, is_patch: bool, show_secrets: bool
) -> List[Line]:
    """
    Return the secrets and the lines with line number.

    :param scan_result: Scan result from the API call
    :param show_secrets: Option to hide secrets value
    :param is_patch: Is the content a patch
    """

    # Patch
    if is_patch:
        return list(get_lines_from_patch(content, filemode))

    # File
    return list(get_lines_from_file(content))


def get_lines_from_file(content: str) -> Iterable[Line]:
    """Return the lines with line number from a file."""
    for line_count, line_content in enumerate(content.split("\n")):
        yield Line(
            content=line_content, category=LineCategory.data, pre_index=line_count + 1
        )


def get_lines_from_patch(content: str, filemode: Filemode) -> Iterable[Line]:
    """Return the lines with line number from a git patch."""
    content += "\n"
    pre_index = 0
    post_index = 0

    for line in content.split("\n"):
        line_type = line[:1]
        line_content = ""
        line_pre_index = None
        line_post_index = None
        category = None

        if line_type == " ":
            line_content = line[1:]
            pre_index += 1
            post_index += 1
            line_pre_index = pre_index
            line_post_index = post_index
        elif line_type == "@":
            m = REGEX_PATCH_HEADER.search(line)
            if m is None:
                continue
            pre_index = int(m.groupdict()["pre_index"])
            post_index = int(m.groupdict()["post_index"])
            line_content = m.groupdict()["line_content"][:-1]

            if filemode == Filemode.NEW or filemode == Filemode.DELETE:
                pre_index = 1
                post_index = 1

            if line_content:
                line_type = " "
                pre_index -= 1
                post_index -= 1
                line_pre_index = None
                line_post_index = None
                category = LineCategory.empty
        elif line_type == "+":
            post_index += 1
            line_post_index = post_index
            line_content = line[1:]
            category = LineCategory.addition
        elif line_type == "-":
            pre_index += 1
            line_pre_index = pre_index
            line_content = line[1:]
            category = LineCategory.deletion
        elif line_type == "\\":
            # This type of line should'nt contain any secret; no need to set indices
            line_content = line[1:]

        if line_type and line_content is not None:
            yield Line(
                content=line_content,
                category=category,
                pre_index=line_pre_index,
                post_index=line_post_index,
            )


class MatchIndices(NamedTuple):
    line_index_start: int
    line_index_end: int
    index_start: int
    index_end: int


def find_match_indices(match: Match, lines: List[Line], is_patch: bool) -> MatchIndices:
    """Utility function.

    Returns a MatchIndices instance where
     - line_index_{start,end} are the indices in the lines of the line objects
       containing the start and end of the match
     - index_{start,end} are the indices of the match in the line_{start,end} objects

    :param match: a Match where index_{start,end} are not None
    :param lines: List of content lines with indices (post_index and pre_index)
    :param is_patch: True if is patch from git, False if file

    :return: MatchIndices
    """
    index = 0
    line_index = 0
    len_line = len(lines[line_index].content) + 1 + int(is_patch)
    # Update line_index until we find the secret start
    while match.index_start >= index + len_line:
        index += len_line
        line_index += 1
        len_line = len(lines[line_index].content) + 1 + int(is_patch)

    line_index_start = line_index
    index_start = match.index_start - index - int(is_patch)

    # Update line_index until we find the secret end
    while match.index_end > index + len_line:
        index += len_line
        line_index += 1
        len_line = len(lines[line_index].content) + 1 + int(is_patch)

    line_index_end = line_index
    index_end = match.index_end - index - int(is_patch) + 1
    return MatchIndices(
        line_index_start,
        line_index_end,
        index_start,
        index_end,
    )


class SupportedCI(Enum):
    GITLAB = "GITLAB"
    TRAVIS = "TRAVIS"
    CIRCLECI = "CIRCLECI"
    JENKINS = "JENKINS HOME"
    GITHUB = "GITHUB ACTIONS"
    BITBUCKET = "BITBUCKET PIPELINES"
    DRONE = "DRONE"
    AZURE = "AZURE PIPELINES"


class SupportedScanMode(Enum):
    REPO = "repo"
    PATH = "path"
    COMMIT_RANGE = "commit_range"
    PRE_COMMIT = "pre_commit"
    PRE_PUSH = "pre_push"
    PRE_RECEIVE = "pre_receive"
    CI = "ci"
    DOCKER = "docker"


json_output_option_decorator = click.option(
    "--json",
    "json_output",
    is_flag=True,
    default=False,
    show_default=True,
    help="JSON output results",
)


def retrieve_client(ctx: click.Context) -> GGClient:
    api_key: Optional[str] = os.getenv("GITGUARDIAN_API_KEY")
    base_uri: str = os.getenv("GITGUARDIAN_API_URL", ctx.obj["config"].api_url)
    if not api_key:
        raise click.ClickException("GitGuardian API Key is needed.")

    session = Session()
    if ctx.obj["config"].allow_self_signed:
        urllib3.disable_warnings()
        session.verify = False

    return GGClient(
        api_key=api_key,
        base_uri=base_uri,
        user_agent="ggshield",
        timeout=60,
        session=session,
    )


def handle_exception(e: Exception, verbose: bool) -> int:
    """
    Handle exception from a scan command.
    """
    if isinstance(e, click.exceptions.Abort):
        return 0
    elif isinstance(e, click.ClickException):
        raise e
    else:
        if verbose:
            traceback.print_exc()
        raise click.ClickException(str(e))



from copy import deepcopy

import pytest

from ggshield.output import TextOutputHandler
from ggshield.scan import Result
from ggshield.scan.scannable import ScanCollection
from ggshield.utils import Filemode
from tests.conftest import (
    _MULTI_SECRET_ONE_LINE_PATCH,
    _MULTI_SECRET_ONE_LINE_PATCH_OVERLAY,
    _MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT,
    _MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT,
    _MULTI_SECRET_TWO_LINES_PATCH,
    _MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT,
    _ONE_LINE_AND_MULTILINE_PATCH_CONTENT,
    _ONE_LINE_AND_MULTILINE_PATCH_SCAN_RESULT,
    _SIMPLE_SECRET_MULTILINE_PATCH,
    _SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT,
    _SIMPLE_SECRET_PATCH,
    _SIMPLE_SECRET_PATCH_SCAN_RESULT,
)


@pytest.mark.parametrize(
    "show_secrets",
    [pytest.param(True, id="show_secrets"), pytest.param(False, id="hide_secrets")],
)
@pytest.mark.parametrize(
    "verbose",
    [pytest.param(True, id="verbose"), pytest.param(False, id="clip_long_lines")],
)
@pytest.mark.parametrize(
    "result_input",
    [
        pytest.param(
            Result(
                content=_SIMPLE_SECRET_PATCH,
                filename="leak.txt",
                filemode=Filemode.NEW,
                scan=_SIMPLE_SECRET_PATCH_SCAN_RESULT,
            ),
            id="_SIMPLE_SECRET_PATCH_SCAN_RESULT",
        ),
        pytest.param(
            Result(
                content=_MULTI_SECRET_ONE_LINE_PATCH,
                filename="leak.txt",
                filemode=Filemode.NEW,
                scan=_MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT,
            ),
            id="_MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT",
        ),
        pytest.param(
            Result(
                content=_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY,
                filename="leak.txt",
                filemode=Filemode.NEW,
                scan=_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT,
            ),
            id="_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT",
        ),
        pytest.param(
            Result(
                content=_MULTI_SECRET_TWO_LINES_PATCH,
                filename="leak.txt",
                filemode=Filemode.NEW,
                scan=_MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT,
            ),
            id="_MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT",
        ),
        pytest.param(
            Result(
                content=_SIMPLE_SECRET_MULTILINE_PATCH,
                filename="leak.txt",
                filemode=Filemode.NEW,
                scan=_SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT,
            ),
            id="_SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT",
        ),
        pytest.param(
            Result(
                content=_ONE_LINE_AND_MULTILINE_PATCH_CONTENT,
                filename="leak.txt",
                filemode=Filemode.NEW,
                scan=_ONE_LINE_AND_MULTILINE_PATCH_SCAN_RESULT,
            ),
            id="_ONE_LINE_AND_MULTILINE_PATCH_CONTENT",
        ),
    ],
)
def test_leak_message(result_input, snapshot, show_secrets, verbose):
    output_handler = TextOutputHandler(show_secrets=show_secrets, verbose=verbose)
    new_result = deepcopy(result_input)
    output = output_handler._process_scan_impl(
        ScanCollection(
            id="scan",
            type="test",
            results=[new_result],
            optional_header="> This is an example header",
        )
    )

    snapshot.assert_match(output)



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot


snapshots = Snapshot()

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT-clip_long_lines-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT-clip_long_lines-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT-verbose-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_OVERLAY_SCAN_RESULT-verbose-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT-clip_long_lines-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT-clip_long_lines-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT-verbose-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_ONE_LINE_PATCH_SCAN_RESULT-verbose-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT-clip_long_lines-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT-clip_long_lines-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT-verbose-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_MULTI_SECRET_TWO_LINES_PATCH_SCAN_RESULT-verbose-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m38d9d3464520ed68f18d16e640a4a8b37ef5b17608b455267d100aa487ead314\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_ONE_LINE_AND_MULTILINE_PATCH_CONTENT-clip_long_lines-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m3\x1b[0m incidents have been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m1945f4a0c42abb19c1a420ddd09b4b4681249a3057c427b95f794b18595e7ffa\x1b[0m) (1 occurrence)

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 2(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m \x1b[0m | \x1b[37m\x1b[22m\x1b[22m@@ -0,0 +1,29 @\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m1\x1b[0m | \x1b[37m\x1b[22m\x1b[22m…**5733 /ce3f9f********************5e43711 \x1b[0m\x1b[91m\x1b[22m\x1b[22m-----BEGIN RSA PRIVATE KEY-----\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m2\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+MIIBOgIBAAJBAIIRkYjxjE3KIZi******************************+******\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m3\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****************************************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m4\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****************************************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m5\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+***********+****************************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m6\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****************+***********************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m7\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+**********************+*****************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m8\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****+******Xme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[91m\x1b[22m\x1b[22m-----END RSA PRIVATE KEY----- \x1b[0m\x1b[37m\x1b[22m\x1b[22mtoken: SG._Yytrtvlj************************…\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m\x1b[93m\x1b[1m\x1b[22m|_________________________________apikey_________________________________|\x1b[0m
\x1b[0m
\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 3(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mSendGrid Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m530e5a4a7ea00814db8845dd0cae5efaa4b974a3ce1c76d0384ba715248a5dc1\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m7\x1b[0m | \x1b[37m\x1b[22m\x1b[22m**********************+*****************************************\x1b[0m
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m8\x1b[0m | \x1b[37m\x1b[22m\x1b[22m****+******Xme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[37m\x1b[22m\x1b[22m…-- token: \x1b[0m\x1b[91m\x1b[22m\x1b[22mSG._Yytrtvlj******************************************-**rRJLGFLBLf0M\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m           \x1b[93m\x1b[1m\x1b[22m|_______________________________apikey______________________________|\x1b[0m
\x1b[0m'''

snapshots['test_leak_message[_ONE_LINE_AND_MULTILINE_PATCH_CONTENT-clip_long_lines-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m3\x1b[0m incidents have been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m1945f4a0c42abb19c1a420ddd09b4b4681249a3057c427b95f794b18595e7ffa\x1b[0m) (1 occurrence)

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 2(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m \x1b[0m | \x1b[37m\x1b[22m\x1b[22m@@ -0,0 +1,29 @\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m1\x1b[0m | \x1b[37m\x1b[22m\x1b[22m…041573 / ce3f9f0362bbe5ab01dfc8ee565e4371 \x1b[0m\x1b[91m\x1b[22m\x1b[22m-----BEGIN RSA PRIVATE KEY-----\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m2\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+MIIBOgIBAAJBAIIRkYjxjE3KIZiEc8k4sWWGNsPYRNE0u0bl5oFVApPLm+uXQ/4l\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m3\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+bKO9LFtMiVPy700oMWLScwAN5OAiqVLMvHUCAwEAAQJANLr8nmEWuV6t2hAwhK5I\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m4\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+NNmBkEo4M/xFxEtl9J7LKbE2gtNrlCQiJlPP1EMhwAjDOzQcJ3lgFB28dkqH5rMW\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m5\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+TQIhANrCE7O+wlCKe0WJqQ3lYlHG91XWyGVgfExJwBDsAD9LAiEAmDY5OSsH0n2A\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m6\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+22tthkAvcN1s66lG+0DztOVJ4QLI2z8CIBPeDGwGpx8pdIicN/5LFuLWbyAcoZaT\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m7\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+bLaA/DCNPniBAiA0l//bzg+M3srIhm04xzLdR9Vb9IjPRlkvN074zdKDVwIhAKJb\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m8\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+RF3C+CMFb0wXme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[91m\x1b[22m\x1b[22m-----END RSA PRIVATE KEY----- \x1b[0m\x1b[37m\x1b[22m\x1b[22mtoken: SG._YytrtvljkWqCrkMa3r5hw.yijiPf2qxr…\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m\x1b[93m\x1b[1m\x1b[22m|_________________________________apikey_________________________________|\x1b[0m
\x1b[0m
\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 3(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mSendGrid Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m530e5a4a7ea00814db8845dd0cae5efaa4b974a3ce1c76d0384ba715248a5dc1\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m7\x1b[0m | \x1b[37m\x1b[22m\x1b[22mbLaA/DCNPniBAiA0l//bzg+M3srIhm04xzLdR9Vb9IjPRlkvN074zdKDVwIhAKJb\x1b[0m
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m8\x1b[0m | \x1b[37m\x1b[22m\x1b[22mRF3C+CMFb0wXme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[37m\x1b[22m\x1b[22m…-- token: \x1b[0m\x1b[91m\x1b[22m\x1b[22mSG._YytrtvljkWqCrkMa3r5hw.yijiPf2qxr2rYArkz3xlLrbv5Zr7-gtrRJLGFLBLf0M\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m           \x1b[93m\x1b[1m\x1b[22m|_______________________________apikey______________________________|\x1b[0m
\x1b[0m'''

snapshots['test_leak_message[_ONE_LINE_AND_MULTILINE_PATCH_CONTENT-verbose-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m3\x1b[0m incidents have been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m1945f4a0c42abb19c1a420ddd09b4b4681249a3057c427b95f794b18595e7ffa\x1b[0m) (1 occurrence)

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 2(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m \x1b[0m | \x1b[37m\x1b[22m\x1b[22m@@ -0,0 +1,29 @\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m1\x1b[0m | \x1b[37m\x1b[22m\x1b[22mFacebookAppKeys:294*********5733 /ce3f9f********************5e43711 \x1b[0m\x1b[91m\x1b[22m\x1b[22m-----BEGIN RSA PRIVATE KEY-----\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m2\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+MIIBOgIBAAJBAIIRkYjxjE3KIZi******************************+******\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m3\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****************************************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m4\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****************************************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m5\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+***********+****************************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m6\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****************+***********************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m7\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+**********************+*****************************************\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m8\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+****+******Xme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[91m\x1b[22m\x1b[22m-----END RSA PRIVATE KEY----- \x1b[0m\x1b[37m\x1b[22m\x1b[22mtoken: SG._Yytrtvlj******************************************-**rRJLGFLBLf0M\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m\x1b[93m\x1b[1m\x1b[22m\x1b[0m
\x1b[0m
\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 3(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mSendGrid Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m530e5a4a7ea00814db8845dd0cae5efaa4b974a3ce1c76d0384ba715248a5dc1\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m7\x1b[0m | \x1b[37m\x1b[22m\x1b[22m**********************+*****************************************\x1b[0m
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m8\x1b[0m | \x1b[37m\x1b[22m\x1b[22m****+******Xme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[37m\x1b[22m\x1b[22m-----END RSA PRIVATE KEY----- token: \x1b[0m\x1b[91m\x1b[22m\x1b[22mSG._Yytrtvlj******************************************-**rRJLGFLBLf0M\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m                                     \x1b[93m\x1b[1m\x1b[22m|_______________________________apikey______________________________|\x1b[0m
\x1b[0m'''

snapshots['test_leak_message[_ONE_LINE_AND_MULTILINE_PATCH_CONTENT-verbose-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m3\x1b[0m incidents have been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mFacebook Access Tokens\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m1945f4a0c42abb19c1a420ddd09b4b4681249a3057c427b95f794b18595e7ffa\x1b[0m) (1 occurrence)

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 2(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m \x1b[0m | \x1b[37m\x1b[22m\x1b[22m@@ -0,0 +1,29 @\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m1\x1b[0m | \x1b[37m\x1b[22m\x1b[22mFacebookAppKeys: 294790898041573 / ce3f9f0362bbe5ab01dfc8ee565e4371 \x1b[0m\x1b[91m\x1b[22m\x1b[22m-----BEGIN RSA PRIVATE KEY-----\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m2\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+MIIBOgIBAAJBAIIRkYjxjE3KIZiEc8k4sWWGNsPYRNE0u0bl5oFVApPLm+uXQ/4l\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m3\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+bKO9LFtMiVPy700oMWLScwAN5OAiqVLMvHUCAwEAAQJANLr8nmEWuV6t2hAwhK5I\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m4\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+NNmBkEo4M/xFxEtl9J7LKbE2gtNrlCQiJlPP1EMhwAjDOzQcJ3lgFB28dkqH5rMW\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m5\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+TQIhANrCE7O+wlCKe0WJqQ3lYlHG91XWyGVgfExJwBDsAD9LAiEAmDY5OSsH0n2A\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m6\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+22tthkAvcN1s66lG+0DztOVJ4QLI2z8CIBPeDGwGpx8pdIicN/5LFuLWbyAcoZaT\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m7\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+bLaA/DCNPniBAiA0l//bzg+M3srIhm04xzLdR9Vb9IjPRlkvN074zdKDVwIhAKJb\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m8\x1b[0m | \x1b[91m\x1b[22m\x1b[22m+RF3C+CMFb0wXme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[91m\x1b[22m\x1b[22m-----END RSA PRIVATE KEY----- \x1b[0m\x1b[37m\x1b[22m\x1b[22mtoken: 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\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m\x1b[93m\x1b[1m\x1b[22m\x1b[0m
\x1b[0m
\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 3(\x1b[93m\x1b[1m\x1b[22mSecrets detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mSendGrid Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m530e5a4a7ea00814db8845dd0cae5efaa4b974a3ce1c76d0384ba715248a5dc1\x1b[0m) (1 occurrence)
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m7\x1b[0m | \x1b[37m\x1b[22m\x1b[22mbLaA/DCNPniBAiA0l//bzg+M3srIhm04xzLdR9Vb9IjPRlkvN074zdKDVwIhAKJb\x1b[0m
\x1b[37m\x1b[22m\x1b[2m \x1b[0m \x1b[37m\x1b[22m\x1b[2m8\x1b[0m | \x1b[37m\x1b[22m\x1b[22mRF3C+CMFb0wXme/ovcDeM1+3W/UmSHYUW4b3WYq4\x1b[0m
\x1b[33m\x1b[22m\x1b[22m \x1b[0m \x1b[33m\x1b[22m\x1b[22m9\x1b[0m | \x1b[37m\x1b[22m\x1b[22m-----END RSA PRIVATE KEY----- token: \x1b[0m\x1b[91m\x1b[22m\x1b[22mSG._YytrtvljkWqCrkMa3r5hw.yijiPf2qxr2rYArkz3xlLrbv5Zr7-gtrRJLGFLBLf0M\x1b[0m
      \x1b[93m\x1b[1m\x1b[22m                                     \x1b[93m\x1b[1m\x1b[22m|_______________________________apikey______________________________|\x1b[0m
\x1b[0m'''

snapshots['test_leak_message[_SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT-clip_long_lines-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT-clip_long_lines-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT-verbose-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_MULTILINE_PATCH_SCAN_RESULT-verbose-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mRSA Private Key\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m060bf63de122848f5efa122fe6cea504aae3b24cea393d887fdefa1529c6a02e\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_PATCH_SCAN_RESULT-clip_long_lines-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mGitHub Token\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m2b5840babacb6f089ddcce1fe5a56b803f8b1f636c6f44cdbf14b0c77a194c93\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_PATCH_SCAN_RESULT-clip_long_lines-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mGitHub Token\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m2b5840babacb6f089ddcce1fe5a56b803f8b1f636c6f44cdbf14b0c77a194c93\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_PATCH_SCAN_RESULT-verbose-hide_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mGitHub Token\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m2b5840babacb6f089ddcce1fe5a56b803f8b1f636c6f44cdbf14b0c77a194c93\x1b[0m) (1 occurrence)
'''

snapshots['test_leak_message[_SIMPLE_SECRET_PATCH_SCAN_RESULT-verbose-show_secrets] 1'] = '''> This is an example header
secrets-engine-version: 2.56.0

🛡️  ⚔️  🛡️  \x1b[94m\x1b[1m\x1b[22m1\x1b[0m incident has been found in file \x1b[93m\x1b[1m\x1b[22mleak.txt\x1b[0m

\x1b[36m\x1b[22m\x1b[22m>>>\x1b[0m Incident 1(\x1b[93m\x1b[1m\x1b[22mSecrets Detection\x1b[0m): \x1b[93m\x1b[1m\x1b[22mGitHub Token\x1b[0m (Ignore with SHA: \x1b[36m\x1b[22m\x1b[22m2b5840babacb6f089ddcce1fe5a56b803f8b1f636c6f44cdbf14b0c77a194c93\x1b[0m) (1 occurrence)
'''



from ast import literal_eval
from typing import Dict, List

import click
from pygitguardian.models import Detail

from ggshield.text_utils import STYLE, display_error, format_text, pluralize


def handle_scan_error(detail: Detail, chunk: List[Dict[str, str]]) -> None:
    if detail.status_code == 401:
        raise click.UsageError(detail.detail)

    details = None

    display_error("\nError scanning. Results may be incomplete.")
    try:
        # try to load as list of dicts to get per file details
        details = literal_eval(detail.detail)
    except Exception:
        pass

    if isinstance(details, list) and details:
        # if the details had per file details
        display_error(
            f"Add the following {pluralize('file', len(details))}"
            " to your paths-ignore:"
        )
        for i, inner_detail in enumerate(details):
            if inner_detail:
                click.echo(
                    f"- {format_text(chunk[i]['filename'], STYLE['filename'])}:"
                    f" {str(inner_detail)}",
                    err=True,
                )
        return
    else:
        # if the details had a request error
        filenames = ", ".join([file_dict["filename"] for file_dict in chunk])
        display_error(
            "The following chunk is affected:\n"
            f"{format_text(filenames, STYLE['filename'])}"
        )

        display_error(str(detail))
